/*
   Project: Adun

   Copyright (C) 2005 Michael Johnston & Jordi Villa-Freixa

   Author: Michael Johnston

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#ifndef _ADUN_MATRIX_
#define _ADUN_MATRIX_
#include <stdio.h>
#include <stdlib.h>

//! \brief Structure for int matrices 
/**
\ingroup Types
*/

typedef struct
{
	int no_rows;
	int no_columns;
	int** matrix;
}
IntMatrix;


//! \brief Structure for float matrices 
/**
\ingroup Types
*/
typedef struct
{
	int no_rows;
	int no_columns;
	float** matrix;
}
FloatMatrix;

//! \brief Structure for double matrices 
/**
\ingroup Types
*/
typedef struct
{
	int no_rows;
	int no_columns;
	double** matrix;
}
DoubleMatrix;

typedef double AdMatrixSize;

//typedefs for AdMatrix - we make AdMatrix refer to the DoubleMatrix type

typedef DoubleMatrix AdMatrix;

//prototypes
/**
\defgroup matrix Matrix
\ingroup Functions
@{
**/

extern void AdSetDoubleMatrixWithValue(DoubleMatrix *, double);
extern void AdSetFloatMatrixWithValue(FloatMatrix *,  float);
extern void AdSetIntMatrixWithValue(IntMatrix *, int);

extern IntMatrix* AdIntMatrixFromRowSection(IntMatrix *, int , int);
extern FloatMatrix* AdFloatMatrixFromRowSection(FloatMatrix *, int , int);
extern DoubleMatrix* AdDoubleMatrixFromRowSection(DoubleMatrix *, int , int);

extern IntMatrix* AdIntMatrixFromRowSelection(IntMatrix *, int* , int);
extern FloatMatrix* AdFloatMatrixFromRowSelection(FloatMatrix *, int* , int);
extern DoubleMatrix* AdDoubleMatrixFromRowSelection(DoubleMatrix *, int* , int);

extern IntMatrix* AdIntMatrixFromColumnSection(IntMatrix *, int , int);
extern FloatMatrix* AdFloatMatrixFromColumnSection(FloatMatrix *, int , int);
extern DoubleMatrix* AdDoubleMatrixFromColumnSection(DoubleMatrix *, int , int);

extern IntMatrix* AdIntMatrixFromColumnSelection(IntMatrix *, int* , int);
extern FloatMatrix* AdFloatMatrixFromColumnSelection(FloatMatrix *, int* , int);
extern DoubleMatrix* AdDoubleMatrixFromColumnSelection(DoubleMatrix *, int* , int);

extern void AdFreeDoubleMatrix(DoubleMatrix*);
extern void AdFreeIntMatrix(IntMatrix*);
extern void AdFreeFloatMatrix(FloatMatrix*);

extern IntMatrix* AdAllocateIntMatrix(int, int);
extern FloatMatrix* AdAllocateFloatMatrix(int, int);
extern DoubleMatrix* AdAllocateDoubleMatrix(int, int);

/**
AdMatrix copy function.
*/
extern void AdCopyAdMatrixToAdMatrix(DoubleMatrix*, DoubleMatrix*);
#endif
